@@ -27,15 +27,18 @@
 #include <KTabWidget>
 #include <KColorScheme>
 
-#include <analitzagui/graph2d.h>
-#include <analitzagui/functionsmodel.h>
 #include <analitzagui/expressionedit.h>
 #include <analitza/analyzer.h>
 #include <analitza/expression.h>
 #include <analitza/variables.h>
 #include <analitza/value.h>
-#include <analitzagui/functionfactory.h>
 #include <analitzagui/algebrahighlighter.h>
+#include <analitzagui/plotsview2d.h>
+#include <analitzaplot/planecurve.h>
+#include <analitzaplot/plotsmodel.h>
+#include <analitzaplot/plotsfactory.h>
+
+using namespace Analitza;
 
 namespace {
 	static const int resolution = 200;
@@ -53,7 +56,7 @@ FunctionEdit::FunctionEdit(QWidget *parent)
 	m_name = new KLineEdit(this);
 	
 	m_func = new ExpressionEdit(this);
-    m_func->setExamples(FunctionFactory::self()->examples());
+    m_func->setExamples(PlotsFactory::self()->examples(Dim2D));
 	m_func->setAns("x");
 	connect(m_func, SIGNAL(textChanged()), this, SLOT(edit()));
 	connect(m_func, SIGNAL(returnPressed()), this, SLOT(ok()));
@@ -75,19 +78,20 @@ FunctionEdit::FunctionEdit(QWidget *parent)
 	m_color->setColor(QColor(0,150,0));
 	connect(m_color, SIGNAL(currentIndexChanged(int)), this, SLOT(colorChange(int)));
 	
-	m_funcsModel=new FunctionsModel;
-	m_funcsModel->setResolution(resolution);
-//	m_funcsModel->addFunction(function(m_name->text(), m_func->expression(), m_color->color()));
+	m_funcsModel=new PlotsModel(this);
+    m_funcsModel->setResolution(resolution);
 	
 	m_viewTabs=new KTabWidget(this);
 	
-	m_graph = new Graph2D(m_funcsModel, m_viewTabs);
+	m_graph = new PlotsView2D(m_viewTabs);
+	m_graph->setModel(m_funcsModel);
 	m_graph->setViewport(QRect(QPoint(-5, 7), QPoint(5, -7)));
 	m_graph->setFocusPolicy(Qt::NoFocus);
 	m_graph->setMouseTracking(false);
 	m_graph->setFramed(true);
 	m_graph->setReadOnly(true);
 	m_graph->setSquares(false);
+	m_graph->setTicksShown(Qt::Orientation(0));
 	
 	m_viewTabs->addTab(m_graph, QIcon::fromTheme("document-preview"), i18n("Preview"));
 	QWidget *options=new QWidget(m_viewTabs);
@@ -151,8 +155,7 @@ void FunctionEdit::setColor(const QColor &newColor)
 {
 	m_color->setColor(newColor);
 	if(m_funcsModel->rowCount()>0)
-		  m_funcsModel->editFunction(0)->setColor(newColor);
-	m_graph->forceRepaint();
+		  m_funcsModel->setData(m_funcsModel->index(0), newColor);
 }
 
 void FunctionEdit::colorChange(int)
@@ -177,9 +180,9 @@ static double calcExp(const Analitza::Expression& exp, Analitza::Variables* v, b
 
 void FunctionEdit::updateUplimit()
 {
-	bool corr = m_uplimit->isCorrect();
-	if(corr) {
-		Analitza::Expression e=m_uplimit->expression();
+	bool corr;
+	Analitza::Expression e=m_uplimit->expression();
+	if(e.isCorrect()) {
 		m_calcUplimit=calcExp(e, m_vars, &corr);
 		m_uplimit->setCorrect(corr);
 		if(corr)
@@ -189,9 +192,9 @@ void FunctionEdit::updateUplimit()
 
 void FunctionEdit::updateDownlimit()
 {
-	bool corr = m_downlimit->isCorrect();
-	if(corr) {
-		Analitza::Expression e=m_downlimit->expression();
+	bool corr;
+	Analitza::Expression e=m_downlimit->expression();
+	if(e.isCorrect()) {
 		m_calcDownlimit=calcExp(e, m_vars, &corr);
 		m_downlimit->setCorrect(corr);
 		if(corr)
@@ -257,34 +260,35 @@ void FunctionEdit::edit()
 		setState(i18n("Downlimit cannot be greater than uplimit"), true);
 		return;
 	}
+	bool added = false;
 	
-	Function f=createFunction();
-	if(f.isCorrect()) {
-		f.setResolution(resolution);
-		f.calc(QPointF());
-	}
+	PlaneCurve* f = 0;
+	PlotBuilder req = PlotsFactory::self()->requestPlot(expression(), Dim2D);
+	if(req.canDraw())
+		f = createFunction();
 	
-	if(f.isCorrect())
-		f.update_points(QRect(-10, 10, 20, -20));
+	if(f && f->isCorrect())
+		f->update(QRect(-10, 10, 20, -20));
 	
-	if(f.isCorrect()) {
+	if(f && f->isCorrect()) {
 		m_funcsModel->clear();
-		m_funcsModel->addFunction(f);
+		m_funcsModel->addPlot(f);
+		added=true;
 		setState(QString("%1:=%2")
-			.arg(m_name->text()).arg(f.expression().toString()), false);
+			.arg(m_name->text()).arg(f->expression().toString()), false);
 	} else {
-		QStringList errors = f.errors();
+		QStringList errors = req.errors();
+		if(f)
+			errors = f->errors();
 		Q_ASSERT(!errors.isEmpty());
 		
 		m_funcsModel->clear();
-		m_graph->forceRepaint();
-// 		m_valid->setText(i18n("<b style='color:red'>WRONG</b>"));
-		
 		setState(errors.first(), true);
 		m_valid->setToolTip(errors.join("<br />"));
+		delete f;
 	}
-	m_func->setCorrect(f.isCorrect());
-	m_ok->setEnabled(f.isCorrect());
+	m_func->setCorrect(added);
+	m_ok->setEnabled(added);
 }
 
 void FunctionEdit::ok()
@@ -298,9 +302,16 @@ void FunctionEdit::focusInEvent(QFocusEvent *)
 	m_func->setFocus();
 }
 
-Function FunctionEdit::createFunction() const
+PlaneCurve* FunctionEdit::createFunction() const
 {
-	return Function(name(), expression(), m_vars, color(), m_calcUplimit, m_calcDownlimit);
+	PlotBuilder req = PlotsFactory::self()->requestPlot(expression(), Dim2D);
+	PlaneCurve* curve = static_cast<PlaneCurve*>(req.create(color(), name(), m_vars));
+	curve->setResolution(resolution);
+	if(m_calcUplimit != m_calcDownlimit) {
+		foreach(const QString& var, curve->parameters())
+			curve->setInterval(var, m_calcUplimit, m_calcDownlimit);
+	}
+	return curve;
 }
 
 Analitza::Expression FunctionEdit::expression() const
